@@ -39,14 +39,21 @@ PAIRTREE_ROOT_DIR = "pairtree_root"
 
 NAMASTE_PATTERN = re.compile(r"[^0=|1=|2=|3=|4=|5=]")  # Must try hard to better this regex
 
+class SiloNotFound(Exception):
+    pass
+
 class Silo(object):
     """Item persistence layer - uses pairtree as a basis for storage."""
     def __init__(self, storage_dir, uri_base=None, **kw):
         self.state = PersistentState()
         self.state['storage_dir'] = storage_dir
         if not uri_base:
-            uri_base = "info:"
-        self.state['uri_base'] = uri_base
+            if self.state.has_key('uri_base') and self.state['uri_base']:
+                uri_base = self.state['uri_base']
+            else:
+                self.state['uri_base'] = uri_base = "info:"
+        else:
+            self.state['uri_base'] = uri_base
         self.state.update(kw)
         self._init_storage()
         
@@ -63,9 +70,9 @@ class Silo(object):
         except OSError:
             logger.error("Cannot make storage directory")
             raise Exception("Cannot make storage directory")
-        except Exception:
-            logger.error("Cannot setup the state persistence file at %s/%s" % (self.state['storage_dir'], PERSISTENCE_FILENAME))
-            raise Exception("Cannot setup the state persistence file at %s/%s" % (self.state['storage_dir'], PERSISTENCE_FILENAME))
+        except Exception, e:
+            logger.error("Cannot setup the state persistence file at %s/%s" % (self.state['storage_dir'], "__manifest.json"))
+            raise Exception("Cannot setup the state persistence file at %s/%s" % (self.state['storage_dir'], "__manifest.json"))
 
     def __iter__(self):
         return self.list_items()
@@ -136,11 +143,30 @@ class Granary(object):
     def _init_granary(self):
         if not path.exists(self.root_dir):
             mkdir(self.root_dir)
+        self.state = PersistentState()
+        self.state.set_filepath(self.root_dir)
         self.silos = [x for x in listdir(self.root_dir) if self.issilo(x)]
         
     def issilo(self, silo_name):
         return path.isdir(path.join(self.root_dir, silo_name)) and path.exists(path.join(self.root_dir, silo_name, 'pairtree_root'))
     
+    def describe_silo(self, silo_name, **kw):
+        if kw:
+            if self.issilo(silo_name):
+                if not self.state.has_key(silo_name):
+                    self.state[silo_name] = {}
+                self.state[silo_name].update(kw)
+            else:
+                raise SiloNotFound
+        else:
+            if self.state.has_key(silo_name):
+                return self.state[silo_name]
+            else:
+                return {}
+    
+    def sync(self):
+        self.state.sync()
+    
     def get_silo(self, silo_name, uri_base=None, **kw):
         return Silo(path.join(self.root_dir, silo_name), uri_base=uri_base, **kw)
         
